@@ -1,5 +1,5 @@
 #' @title Plot map tiles
-#' @description Plot a SpatRaster object over a map. It can be used to plot tiles.
+#' @description Plot map tiles.
 #' @name plot_tiles
 #' @param x a SpatRaster object.
 #' @param add whether to add the layer to an existing plot (TRUE) or
@@ -17,17 +17,18 @@
 #' @examples
 #' library(sf)
 #' library(maptiles)
-#' nc <- st_read(system.file("shape/nc.shp", package="sf"), quiet = TRUE)
+#' nc <- st_read(system.file("shape/nc.shp", package = "sf"), quiet = TRUE)
 #' nc_osm <- get_tiles(nc, crop = TRUE)
 #' plot_tiles(nc_osm)
 plot_tiles <- function(x, adjust = FALSE, add = FALSE, ...) {
-  if(is.null(x)){
+  if (is.null(x)) {
     message("x is NULL")
     return(invisible(NULL))
   }
-  if (!inherits(x, 'SpatRaster')){
+  if (!inherits(x, "SpatRaster")) {
     warning(paste0("x should be a SpatRaster"),
-            call. = FALSE)
+      call. = FALSE
+    )
     return(invisible(NULL))
   }
   ops <- list(...)
@@ -41,23 +42,25 @@ plot_tiles <- function(x, adjust = FALSE, add = FALSE, ...) {
 
   # Add margins if the raster is smaller than the device
   # Zoom-in if the raster is larger than the device
-  if(adjust == TRUE && add == FALSE){
+  if (adjust == TRUE && add == FALSE) {
     tsp <- dim(ops$x)[2:1]
     dsp <- dev.size("px")
     dsi <- dev.size("in")
-    dd <- ((dsp - tsp)/ 2) / (dsp/dsi)
+    dd <- ((dsp - tsp) / 2) / (dsp / dsi)
     dd <- c(dd[2:1], dd[2:1]) / 0.2
-    if(min(dd)>=0){
+    if (min(dd) >= 0) {
       ops$mar <- dd
-    }else{
+    } else {
       et <- terra::ext(ops$x)
       rt <- terra::res(ops$x)[1]
       wp <- (tsp[1] - dsp[1]) / 2
       hp <- (tsp[2] - dsp[2]) / 2
-      et[1:4] <- c(et[1] + wp * rt,
-                   et[2] - wp * rt,
-                   et[3] + hp * rt,
-                   et[4] - hp * rt)
+      et[1:4] <- c(
+        et[1] + wp * rt,
+        et[2] - wp * rt,
+        et[3] + hp * rt,
+        et[4] - hp * rt
+      )
       ops$ext <- et
     }
     ops$smooth <- FALSE
